package es.uca.xmleye.gui;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.ResourceBundle;
import java.util.Locale;

import es.uca.xmleye.gui.util.AccionClick;
import es.uca.xmleye.gui.util.AccionResource;
import es.uca.xmleye.presentacion.ModPresentacionVentanaTipos;
import es.uca.xmleye.tipos.ExcepcionTipo;

/**
 * Diálogo de tipos de documentos aceptados.
 * 
 * Este diálogo permite definir todos los tipos de documentos que no son .xml
 * que acepta el visor. De tal forma, se puede añadir soporte para abrir (a
 * través de un programa de preprocesado) documentos Lisp con demostraciones de
 * ACL2 o ficheros de texto con demostraciones de Otter, etc.
 * 
 * $Id$
 * 
 * @author Antonio García Domínguez
 * @version 1.0
 */
public class VentanaTipos extends JDialog {
	/**
	 * 
	 */
	private static final long serialVersionUID = -8279336889956466721L;
	private static final int ANCHO_MIN = 500;
	private static final int ALTO_MIN = 400;

	// Recursos para internacionalización
	private static final ResourceBundle res;

	// Modelo de presentación
	ModPresentacionVentanaTipos modelo;

	// Controles de importancia de la UI
	JButton cmdAceptar;
	JButton cmdCancelar;
	JButton cmdRestaurar;
	JList listaTipos;
	JTextField txtExtensiones;
	JTextField txtNombre;
	JTextField txtOrdenImportacion;
	JTextField txtOrdenEdicion;

	static {
		res = ResourceBundle.getBundle(
				"es.uca.xmleye.gui.l10n.ResVentanaTipos", Locale.getDefault());
	}

	public VentanaTipos(JFrame padre, ModPresentacionVentanaTipos modelo) {
		super(padre, res.getString("wnd_types"), true);

		this.modelo = modelo;

		inicPropiedades();
		inicComponentes();
		inicAccesos();
		inicEventos();

		sincronizar();

		pack();
		setLocationRelativeTo(padre);
		setVisible(true);
	}

	/**
	 * Programa básico de prueba.
	 * 
	 * Nos limitamos a abrir el diálogo de la manera más básica posible. Se usa
	 * para poder probarlo de forma separada al resto de la aplicación.
	 * 
	 * SÓLO PARA PRUEBAS.
	 * 
	 * @param args
	 *            Argumentos pasados por la línea de órdenes.
	 */
	public static void main(String[] args) throws Exception {
		new VentanaTipos(null, new ModPresentacionVentanaTipos(
				new String[] { "tipos" }));
	}

	/** METODOS PROTEGIDOS * */

	private void inicPropiedades() {
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		final JPanel panel = new JPanel(new BorderLayout());
		panel.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));
		setContentPane(panel);

		setMinimumSize(new Dimension(ANCHO_MIN, ALTO_MIN));
	}

	private void inicComponentes() {
		listaTipos = new JList(modelo.getModeloLista());
		JScrollPane panelListaExt = new JScrollPane(listaTipos);
		cmdRestaurar = new JButton(new AccionRestaurar());

		txtOrdenImportacion = crearCampoTexto();
		txtOrdenEdicion = crearCampoTexto();
		txtNombre = crearCampoTexto();
		txtExtensiones = crearCampoTexto();

		JLabel lblOrdenImportacion = crearEtiqueta(txtOrdenImportacion,
				"lbl_opencmd");
		JLabel lblOrdenEdicion = crearEtiqueta(txtOrdenEdicion, "lbl_editcmd");
		JLabel lblNombre = crearEtiqueta(txtNombre, "lbl_typename");
		JLabel lblExtensiones = crearEtiqueta(txtExtensiones, "lbl_extensions");

		JPanel panelCmdExtensiones = new JPanel();
		panelCmdExtensiones.setLayout(new FlowLayout(FlowLayout.CENTER));
		panelCmdExtensiones.add(cmdRestaurar);

		JPanel panelExtensiones = new JPanel();
		panelExtensiones.setLayout(new BorderLayout());
		panelExtensiones.add(panelListaExt, BorderLayout.CENTER);
		panelExtensiones.add(panelCmdExtensiones, BorderLayout.SOUTH);
		panelExtensiones.setPreferredSize(new Dimension(200, 150));
		panelExtensiones.setMaximumSize(new Dimension(200, 1500));
		panelExtensiones.setBorder(BorderFactory.createEmptyBorder(3, 2, 2, 5));

		JPanel panelCampos = new JPanel();
		panelCampos.setLayout(new BoxLayout(panelCampos, BoxLayout.Y_AXIS));
		panelCampos.add(lblNombre);
		panelCampos.add(txtNombre);
		panelCampos.add(lblOrdenImportacion);
		panelCampos.add(txtOrdenImportacion);
		panelCampos.add(lblOrdenEdicion);
		panelCampos.add(txtOrdenEdicion);
		panelCampos.add(lblExtensiones);
		panelCampos.add(txtExtensiones);

		JPanel panelContenido = new JPanel();
		panelContenido
				.setLayout(new BoxLayout(panelContenido, BoxLayout.X_AXIS));
		panelContenido.add(panelExtensiones);
		panelContenido.add(panelCampos);
		getContentPane().add(panelContenido, BorderLayout.CENTER);

		JPanel panelBotones = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		cmdAceptar = new JButton(new AccionAceptar());
		cmdCancelar = new JButton(new AccionCancelar());
		panelBotones.add(cmdAceptar);
		panelBotones.add(cmdCancelar);
		getContentPane().add(panelBotones, BorderLayout.SOUTH);
	}

	private void inicAccesos() {
		// Aceptar al pulsar INTRO
		getRootPane().setDefaultButton(cmdAceptar);

		// Cancelar al pulsar ESCAPE
		cmdCancelar.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				KeyStroke.getKeyStroke("ESCAPE"), "click");
		cmdCancelar.getActionMap().put("click", new AccionClick(cmdCancelar));
	}

	private void inicEventos() {
		listaTipos.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				modelo.seleccionCambiada(listaTipos.getSelectedIndex());
				sincronizar();
			}
		});

		txtOrdenImportacion.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				modelo.textoOrdenImportacionCambiado(txtOrdenImportacion
						.getText());
				sincronizar();
			}
		});

		txtOrdenEdicion.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				modelo.textoOrdenEdicionCambiado(txtOrdenEdicion.getText());
				sincronizar();
			}
		});

		txtNombre.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				modelo.textoNombreCambiado(txtNombre.getText());
				sincronizar();
			}
		});

		txtExtensiones.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				modelo.textoExtensionesCambiado(txtExtensiones.getText());
				sincronizar();
			}
		});
	}

	/** METODOS PRIVADOS * */

	/**
	 * Crea una etiqueta para un objeto dado.
	 * 
	 * @param comp
	 *            Componente para el que es la etiqueta.
	 * @param clave
	 *            Clave dentro del ResourceBundle del título: [clave]_m es el
	 *            mnemónico.
	 * @return Etiqueta con el título, asociación y mnemónico listos.
	 */
	private static JLabel crearEtiqueta(Component comp, String clave) {
		final JLabel etiq = new JLabel(res.getString(clave));
		etiq.setLabelFor(comp);
		etiq.setDisplayedMnemonic(((Character) res.getObject(clave + "_m"))
				.charValue());
		return etiq;
	}

	/**
	 * Crea un campo de texto con el tamaño correcto.
	 * 
	 * @return Campo de texto con las propiedades ya establecidas.
	 */
	private static JTextField crearCampoTexto() {
		final JTextField campo = new JTextField();
		campo.setPreferredSize(new Dimension(150, 20));
		campo.setMaximumSize(new Dimension(1500, 20));
		return campo;
	}

	/** Sincroniza la interfaz con el modelo de presentación. */
	private void sincronizar() {
		txtOrdenImportacion.setText(modelo.getTextoOrdenImportacion());
		txtOrdenEdicion.setText(modelo.getTextoOrdenEdicion());
		txtNombre.setText(modelo.getTextoNombre());
		txtExtensiones.setText(modelo.getTextoExtensiones());

		listaTipos.setSelectedIndex(modelo.getSeleccion());
	}

	/** CLASES INTERNAS * */

	private class AccionAceptar extends AbstractAction {
		/**
		 * 
		 */
		private static final long serialVersionUID = -4885620202351111999L;

		public AccionAceptar() {
			super(res.getString("cmd_ok"));
			putValue(SHORT_DESCRIPTION, res.getString("cmd_ok_d"));
		}

		public void actionPerformed(ActionEvent e) {
			try {
				modelo.cambiosConfirmados();
				VentanaTipos.this.dispose();
			} catch (ExcepcionTipo ex) {
				ex.printStackTrace();
				GestorMensajesSingleton.getInstancia().mostrarError(
						ex.getMessage());
			}
		}
	}

	private class AccionCancelar extends AbstractAction {
		/**
		 * 
		 */
		private static final long serialVersionUID = -5848723851177427007L;

		public AccionCancelar() {
			super(res.getString("cmd_cancel"));
			putValue(SHORT_DESCRIPTION, res.getString("cmd_cancel_d"));
		}

		public void actionPerformed(ActionEvent e) {
			try {
				modelo.cambiosCancelados();
			} catch (ExcepcionTipo ex) {
				ex.printStackTrace();
				GestorMensajesSingleton.getInstancia().mostrarError(
						ex.getMessage());
			} finally {
				VentanaTipos.this.dispose();
			}
		}
	}

	private class AccionRestaurar extends AccionResource {
		/**
		 * 
		 */
		private static final long serialVersionUID = 696928057301305233L;

		public AccionRestaurar() {
			super(res, "cmd_restore");
		}

		public void actionPerformed(ActionEvent e) {
			try {
				modelo.restauracionSolicitada();
				sincronizar();
			} catch (ExcepcionTipo ex) {
				ex.printStackTrace();
				GestorMensajesSingleton.getInstancia().mostrarError(
						ex.getMessage());
			}
		}
	}
}