package es.uca.xmleye.gui.html;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.html.*;

public class ObservadorLinkCursor implements HyperlinkListener {
	private Container contenedor;
	private Cursor cursorLink;

	/**
	 * Constructor con cursor predefinido, asumiendo que el EditorPane tiene
	 * instalado ya un HTMLEditorKit.
	 * 
	 * @param c
	 *            Contenedor cuyo cursor se va a modificar.
	 * @param e
	 *            EditorPane dentro del cual se hallan las referencias.
	 */
	public ObservadorLinkCursor(Container c, JEditorPane e) {
		this(c, ((HTMLEditorKit) e.getEditorKit()).getLinkCursor());
	}

	/**
	 * Constructor con cursor personalizado.
	 * 
	 * @param c
	 *            Contenedor cuyo cursor se va a modificar.
	 * @param cursorLink
	 *            Cursor a mostrar al mover el ratón por encima de un enlace.
	 */
	public ObservadorLinkCursor(Container c, Cursor cursorLink) {
		this.contenedor = c;
		this.cursorLink = cursorLink;
	}

	public void hyperlinkUpdate(HyperlinkEvent e) {
		if (e.getEventType() == HyperlinkEvent.EventType.ENTERED) {
			contenedor.setCursor(cursorLink);
		} else {
			contenedor.setCursor(Cursor.getDefaultCursor());
		}
	}
}
