package es.uca.xmleye.util;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.util.ArrayList;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

/**
 * Prepara las órdenes para su ejecución, realizando todas las sustituciones y
 * demás cambios necesarios sobre la <code>String</code> original.
 */
public class ConstructorOrdenes {

	/**
	 * <p>
	 * Crea una lista con el ejecutable y sus parámetros. Antes ha realizado los
	 * reemplazos de toda cadena en <code>claves</code> por su correspondiente
	 * en <code>valores</code>.
	 * </p>
	 * 
	 * <p>
	 * Se asume que <code>claves.length == valores.length</code>. De lo
	 * contrario, el comportamiento es indefinido.
	 * </p>
	 * 
	 * @param orden
	 *            Orden a ejecutar.
	 * @param claves
	 *            Claves a reemplazar.
	 * @param valores
	 *            Valores a usar para los reemplazos.
	 * @return Lista donde cada elemento es el ejecutable o uno de los
	 *         parámetros, tras realizar la separación por espacios en blanco y
	 *         las sustituciones.
	 */
	public static ArrayList<String> prepararOrden(String orden,
			String[] claves, String[] valores) {
		for (int i = 0; i < claves.length; ++i) {
			orden = orden.replace(claves[i], valores[i]);
		}

		Matcher tokenMatcher = Pattern.compile("\"(.*?)\"|\\S+").matcher(orden);

		ArrayList<String> listTokens = new ArrayList<String>();
		while (tokenMatcher.find()) {
			listTokens.add(tokenMatcher.group(1) != null ? tokenMatcher
					.group(1) : tokenMatcher.group());
		}

		return listTokens;
	}
}
