package es.uca.xmleye.xpath;

/*
  Copyright (C) 2005-2008 Antonio García Domínguez
  
  This program is free software; you can redistribute it 
  and/or modify it under the terms of the GNU General Public 
  License as published by the Free Software Foundation; 
  either version 2 of the License, or (at your option) any 
  later version.
  
  This program is distributed in the hope that it will be 
  useful, but WITHOUT ANY WARRANTY; without even the implied 
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
  PURPOSE. See the GNU General Public License for more 
  details.
  
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 59 Temple Place, Suite 330, 
  Boston, MA 02111-1307 USA
*/

import java.util.LinkedList;
import java.io.*;
import org.w3c.dom.*;
import org.xml.sax.*;

import es.uca.xmleye.xpath.GestorRutasXPath;

import javax.xml.parsers.*;
import javax.xml.xpath.*;
import junit.framework.*;
import junit.textui.*;

/** 
    Pruebas de unidad para la clase GestorRutasXPath.
*/
public class PruebaGestorRutasXPath extends TestCase
{
  private Document _docXML;
  private XPath _xpath;
  
  /** 
      <p>Barremos el árbol, comprobando que todo nodo es accesible
      mediante la ruta XPath que generamos.</p>

      <p>Por supuesto, tenemos que saltarnos el nodo DOCTYPE: aunque
      presente en el árbol XML, no aparece en el modelo de datos usado
      por XPath. Se espera que GestorRutasXPath haga lo mismo.</p>

      @see <a href="http://www.w3.org/TR/xpath#data-modelo">XPath Data Model</a>
  */
  public void testNodoAXPath() 
  {
    final LinkedList<Node> pilaRecorrido = new LinkedList<Node>();
    final Node raiz = (Node)_docXML;
    pilaRecorrido.addLast(raiz);
    
    try {
      while (!pilaRecorrido.isEmpty()) {
	final Node nodo      = pilaRecorrido.removeLast();
	final String camino  = GestorRutasXPath.obtenerRuta(nodo, raiz);
	final Node resultado = (Node)_xpath.evaluate(camino, raiz, XPathConstants.NODE);
	final Node resAlternativo = GestorRutasXPath.obtenerNodo(camino, raiz).item(0);

	if (resultado == null || !resultado.equals(nodo) || !resAlternativo.equals(nodo)) {
	  fail("La ruta " + camino + " es incorrecta para el nodo " + nodo + ".\n"
               + "XPath -> " + resultado + "\n"
               + "GestorRutasXPath -> " + resAlternativo);
	}

	final NodeList hijos = nodo.getChildNodes();
	for (int i = 0; i < hijos.getLength(); ++i) {
	  if (hijos.item(i).getNodeType() != Node.DOCUMENT_TYPE_NODE) {
	    // Excluimos los nodos DOCTYPE
	    pilaRecorrido.add(hijos.item(i));
	  }
	}
      }
    } catch (XPathExpressionException xpathEx) {
      fail("No se ha podido evaluar la ruta: " + xpathEx.toString());
    }
  }

  /** Crea el conjunto de casos de prueba para JUnit. */
  public static Test suite() 
  {
    return new TestSuite(PruebaGestorRutasXPath.class);
  }

  /** Lanza esta colección de casos de prueba mediante JUnit. */
  public static void main(String[] args) 
  {
    TestRunner.run(suite());
  }

  /** Preparación: cargamos un documento XML para pruebas. */
  protected void setUp() 
    throws FileNotFoundException, 
    ParserConfigurationException, 
    SAXException, IOException
  {
    final InputStream fileIS 
      = new FileInputStream("build/es/uca/xmleye/xpath/PruebaGestorRutasXPath.xml");
    final InputStream bufferIS
      = new BufferedInputStream(fileIS);
    _docXML
      = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(bufferIS);

    _xpath 
      = XPathFactory.newInstance().newXPath();
  }

  /** Libera los recursos necesarios para hacer las pruebas. */
  protected void tearDown() 
  {
    _docXML = null;
  }
}
